@@ -54,7 +54,7 @@ def execute_concurrent(session, statements_and_parameters, concurrency=100, rais
         statements_and_params = []
         for user_id in user_ids:
             params = (user_id, )
-            statatements_and_params.append((select_statement, params))
+            statements_and_params.append((select_statement, params))
 
         results = execute_concurrent(
             session, statements_and_params, raise_on_first_error=False)
@@ -73,7 +73,7 @@ def execute_concurrent(session, statements_and_parameters, concurrency=100, rais
         return []
 
     # TODO handle iterators and generators naturally without converting the
-    # whole thing to a list.  This would requires not building a result
+    # whole thing to a list.  This would require not building a result
     # list of Nones up front (we don't know how many results there will be),
     # so a dict keyed by index should be used instead.  The tricky part is
     # knowing when you're the final statement to finish.
@@ -83,7 +83,7 @@ def execute_concurrent(session, statements_and_parameters, concurrency=100, rais
     first_error = [] if raise_on_first_error else None
     to_execute = len(statements_and_parameters)
     results = [None] * to_execute
-    num_finished = count(start=1)
+    num_finished = count(1)
     statements = enumerate(iter(statements_and_parameters))
     for i in xrange(min(concurrency, len(statements_and_parameters))):
         _execute_next(_sentinel, i, event, session, statements, results, None, num_finished, to_execute, first_error)
@@ -110,7 +110,7 @@ def execute_concurrent_with_args(session, statement, parameters, *args, **kwargs
 
         statement = session.prepare("INSERT INTO mytable (a, b) VALUES (1, ?)")
         parameters = [(x,) for x in range(1000)]
-        execute_concurrent_with_args(session, statement, parameters)
+        execute_concurrent_with_args(session, statement, parameters, concurrency=50)
     """
     return execute_concurrent(session, list(zip(cycle((statement,)), parameters)), *args, **kwargs)
 
