@@ -11,7 +11,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 import atexit
 from collections import deque
 from functools import partial
@@ -21,12 +20,11 @@ import socket
 from threading import Event, Lock, Thread
 import weakref
 
-from six import BytesIO
+from six.moves import xrange
 
 from cassandra import OperationTimedOut
 from cassandra.connection import Connection, ConnectionShutdown, NONBLOCKING
 from cassandra.protocol import RegisterMessage
-from cassandra.marshal import int32_unpack
 try:
     import cassandra.io.libevwrapper as libev
 except ImportError:
@@ -59,6 +57,7 @@ def _cleanup(loop_weakref):
 class LibevLoop(object):
 
     def __init__(self):
+        self._pid = os.getpid()
         self._loop = libev.Loop()
         self._notifier = libev.Async(self._loop)
         self._notifier.start()
@@ -125,6 +124,15 @@ class LibevLoop(object):
         if not self._thread:
             return
 
+        for conn in self._live_conns | self._new_conns | self._closed_conns:
+            conn.close()
+            if conn._write_watcher:
+                conn._write_watcher.stop()
+                del conn._write_watcher
+            if conn._read_watcher:
+                conn._read_watcher.stop()
+                del conn._read_watcher
+
         log.debug("Waiting for event loop thread to join...")
         self._thread.join(timeout=1.0)
         if self._thread.is_alive():
@@ -133,6 +141,7 @@ class LibevLoop(object):
                 "Please call Cluster.shutdown() to avoid this.")
 
         log.debug("Event loop thread was joined")
+        self._loop = None
 
     def connection_created(self, conn):
         with self._conn_set_lock:
@@ -215,6 +224,17 @@ class LibevConnection(Connection):
     def initialize_reactor(cls):
         if not cls._libevloop:
             cls._libevloop = LibevLoop()
+        else:
+            if cls._libevloop._pid != os.getpid():
+                log.debug("Detected fork, clearing and reinitializing reactor state")
+                cls.handle_fork()
+                cls._libevloop = LibevLoop()
+
+    @classmethod
+    def handle_fork(cls):
+        if cls._libevloop:
+            cls._libevloop._cleanup()
+            cls._libevloop = None
 
     @classmethod
     def factory(cls, *args, **kwargs):
@@ -233,19 +253,29 @@ class LibevConnection(Connection):
         Connection.__init__(self, *args, **kwargs)
 
         self.connected_event = Event()
-        self._iobuf = BytesIO()
 
         self._callbacks = {}
         self.deque = deque()
         self._deque_lock = Lock()
 
-        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        if self.ssl_options:
-            if not ssl:
-                raise Exception("This version of Python was not compiled with SSL support")
-            self._socket = ssl.wrap_socket(self._socket, **self.ssl_options)
-        self._socket.settimeout(1.0)  # TODO potentially make this value configurable
-        self._socket.connect((self.host, self.port))
+        sockerr = None
+        addresses = socket.getaddrinfo(self.host, self.port, socket.AF_UNSPEC, socket.SOCK_STREAM)
+        for (af, socktype, proto, canonname, sockaddr) in addresses:
+            try:
+                self._socket = socket.socket(af, socktype, proto)
+                if self.ssl_options:
+                    if not ssl:
+                        raise Exception("This version of Python was not compiled with SSL support")
+                    self._socket = ssl.wrap_socket(self._socket, **self.ssl_options)
+                self._socket.settimeout(1.0)  # TODO potentially make this value configurable
+                self._socket.connect(sockaddr)
+                sockerr = None
+                break
+            except socket.error as err:
+                sockerr = err
+        if sockerr:
+            raise socket.error(sockerr.errno, "Tried connecting to %s. Last error: %s" % ([a[4] for a in addresses], sockerr.strerror))
+
         self._socket.setblocking(0)
 
         if self.sockopts:
@@ -335,37 +365,7 @@ class LibevConnection(Connection):
                 return
 
         if self._iobuf.tell():
-            while True:
-                pos = self._iobuf.tell()
-                if pos < 8 or (self._total_reqd_bytes > 0 and pos < self._total_reqd_bytes):
-                    # we don't have a complete header yet or we
-                    # already saw a header, but we don't have a
-                    # complete message yet
-                    break
-                else:
-                    # have enough for header, read body len from header
-                    self._iobuf.seek(4)
-                    body_len = int32_unpack(self._iobuf.read(4))
-
-                    # seek to end to get length of current buffer
-                    self._iobuf.seek(0, os.SEEK_END)
-                    pos = self._iobuf.tell()
-
-                    if pos >= body_len + 8:
-                        # read message header and body
-                        self._iobuf.seek(0)
-                        msg = self._iobuf.read(8 + body_len)
-
-                        # leave leftover in current buffer
-                        leftover = self._iobuf.read()
-                        self._iobuf = BytesIO()
-                        self._iobuf.write(leftover)
-
-                        self._total_reqd_bytes = 0
-                        self.process_msg(msg, body_len)
-                    else:
-                        self._total_reqd_bytes = body_len + 8
-                        break
+            self.process_io_buffer()
         else:
             log.debug("Connection %s closed by server", self)
             self.close()
