@@ -99,7 +99,17 @@ def named_tuple_factory(colnames, rows):
     .. versionchanged:: 2.0.0
         moved from ``cassandra.decoder`` to ``cassandra.query``
     """
-    Row = namedtuple('Row', map(_clean_column_name, colnames))
+    clean_column_names = map(_clean_column_name, colnames)
+    try:
+        Row = namedtuple('Row', clean_column_names)
+    except:
+        log.warn("Failed creating named tuple for results with column names %s (cleaned: %s) (see Python 'namedtuple' documentation for details on name rules). "
+                 "Results will be returned with positional names. "
+                 "Avoid this by choosing different names, using SELECT \"<col name>\" AS aliases, "
+                 "or specifying a different row_factory on your Session" %
+                 (colnames, clean_column_names))
+        Row = namedtuple('Row', clean_column_names, rename=True)
+
     return [Row(*row) for row in rows]
 
 
@@ -228,9 +238,9 @@ class Statement(object):
         self._serial_consistency_level = None
 
     serial_consistency_level = property(
-         _get_serial_consistency_level,
-         _set_serial_consistency_level,
-         _del_serial_consistency_level,
+        _get_serial_consistency_level,
+        _set_serial_consistency_level,
+        _del_serial_consistency_level,
         """
         The serial consistency level is only used by conditional updates
         (``INSERT``, ``UPDATE`` and ``DELETE`` with an ``IF`` condition).  For
@@ -356,9 +366,8 @@ class PreparedStatement(object):
                 try:
                     routing_key_indexes = [statement_indexes[c.name]
                                            for c in partition_key_columns]
-                except KeyError:
-                    pass  # we're missing a partition key component in the prepared
-                          # statement; just leave routing_key_indexes as None
+                except KeyError:  # we're missing a partition key component in the prepared
+                    pass          # statement; just leave routing_key_indexes as None
 
         return PreparedStatement(column_metadata, query_id, routing_key_indexes,
                                  query, keyspace, protocol_version)
@@ -641,6 +650,7 @@ class BatchStatement(Statement):
         elif isinstance(statement, PreparedStatement):
             query_id = statement.query_id
             bound_statement = statement.bind(() if parameters is None else parameters)
+            self._maybe_set_routing_key(bound_statement)
             self._statements_and_parameters.append(
                 (True, query_id, bound_statement.values))
         elif isinstance(statement, BoundStatement):
@@ -648,6 +658,7 @@ class BatchStatement(Statement):
                 raise ValueError(
                     "Parameters cannot be passed with a BoundStatement "
                     "to BatchStatement.add()")
+            self._maybe_set_routing_key(statement)
             self._statements_and_parameters.append(
                 (True, statement.prepared_statement.query_id, statement.values))
         else:
@@ -656,6 +667,7 @@ class BatchStatement(Statement):
             if parameters:
                 encoder = Encoder() if self._session is None else self._session.encoder
                 query_string = bind_params(query_string, parameters, encoder)
+            self._maybe_set_routing_key(statement)
             self._statements_and_parameters.append((False, query_string, ()))
         return self
 
@@ -668,6 +680,10 @@ class BatchStatement(Statement):
         for statement, value in zip(statements, parameters):
             self.add(statement, parameters)
 
+    def _maybe_set_routing_key(self, statement):
+        if self.routing_key is None and statement.routing_key is not None:
+            self.routing_key = statement.routing_key
+
     def __str__(self):
         consistency = ConsistencyLevel.value_to_name.get(self.consistency_level, 'Not Set')
         return (u'<BatchStatement type=%s, statements=%d, consistency=%s>' %
